@@ -8,6 +8,7 @@ use warnings;
 use File::Find::Rule;
 use File::Basename;
 use File::Spec;
+use IO::File;
 use JSON;
 use Path::Class ();
 use XML::LibXML;
@@ -15,7 +16,7 @@ use XML::LibXML::PrettyPrint;
 
 use OTRS::OPM::Maker -command;
 
-our $VERSION = 0.02;
+our $VERSION = 1.06;
 
 sub abstract {
     return "build sopm file based on metadata";
@@ -39,7 +40,7 @@ sub validate_args {
 
         @json_files > 1 ?
             $self->usage_error( 'found more than one json file, please specify the config file to use' ) :
-            do{ $opt->{config} = $json_files[0];
+            do{ $opt->{config} = $json_files[0] };
     }
     
     my $config = Path::Class::File->new( $opt->{config} );
@@ -61,7 +62,7 @@ sub execute {
 
     # check needed info
     for my $needed (qw(name version framework)) {
-        if ( !$json->{$name} ) {
+        if ( !$json->{$needed} ) {
             print STDERR "Need $needed in config file";
             exit 1;
         }
@@ -107,13 +108,13 @@ sub execute {
 
         # remove "hidden" files from list;
         @files = grep{ 
-            ( substr( $_, 0, 1 ) eq '.' ) &&
+            ( substr( $_, 0, 1 ) ne '.' ) &&
             $_ !~ m{[\\/]\.} 
-        }@files;
+        }sort @files;
 
         push @xml_parts, 
             sprintf "    <Filelist>\n%s\n    </Filelist>",
-                join "\n", map{ my $permission = $_ =~ /^bin/ ? 755 : 644; qq~     <File Permission="$permission" Location="$_" />~ }@files;
+                join "\n", map{ my $permission = $_ =~ /^bin/ ? 755 : 644; qq~        <File Permission="$permission" Location="$_" />~ }@files;
     }
 
     my %actions = (
@@ -122,41 +123,73 @@ sub execute {
         Upgrade   => 'post',
     );
 
-    my %subaction_code = (
+    my %action_code = (
         TableCreate => \&_TableCreate,
         Insert      => \&_Insert,
         TableDrop   => \&_TableDrop,
     );
+    
+    my %tables_to_delete;
+    my %db_actions;
 
     ACTION:
     for my $action ( @{ $json->{database} || [] } ) {
-        next ACTION if !$actions{ $action->{type} };
+        my $tmp_version = $action->{version};
+        my @versions    = ref $tmp_version ? @{$tmp_version} : ($tmp_version);
 
-        my $action_type = $action->{type};
-        my $order       = $actions{ $action->{type} };
+        VERSION:
+        for my $version ( @versions ) {
+            my $action_type = $version ? 'Upgrade' : 'Install';
+            my $op          = $action->{type};
 
-        my @actions;
-
-        SUBACTION:
-        for my $subaction ( @{ $action->{actions} || [] } ) {
-            my $type    = $subaction->{type};
-
-            next SUBACTION if !$subaction_code{$type};
-            push @actions, $subaction_code{$type};
+            next VERSION if !$action_code{$op};
+            
+            if ( $op eq 'TableCreate' ) {
+                my $table = $action->{name};
+                $tables_to_delete{$table}++;
+            }
+            elsif ( $op eq 'TableDrop' ) {
+                my $table = $action->{name};
+                delete $tables_to_delete{$table};
+            }
+        
+            $action->{version} = $version;    
+            push @{ $db_actions{$action_type} }, $action_code{$op}->($action);
         }
-
+    }
+    
+    for my $action_type ( qw/Install Upgrade/ ) {
+        
+        next if !$db_actions{$action_type};
+        
+        my $order = $actions{$action_type};
+        
         push @xml_parts,
             sprintf qq~    <Database$action_type Type="$order">
 %s
-    </Database$action_type>~, join "\n", @actions;
+    </Database$action_type>~, join "\n", @{ $db_actions{$action_type} };
+    }
+    
+    if ( %tables_to_delete ) {
+        my @actions;
+        
+        for my $table ( keys %tables_to_delete ) {
+            push @actions, _TableDrop({ name => $table });
+        }
+        
+        push @xml_parts,
+            sprintf qq~    <DatabaseUninstall Type="pre">
+%s
+    </DatabaseUninstall>~, join "\n", @actions;
     }
 
     for my $code ( @{ $json->{code} || [] } ) {
+        $code->{type} = 'Code' . $code->{type};
         push @xml_parts, _CodeTemplate( $code->{type}, $code->{version}, $code->{function} || $code->{type} );
     }
     
     my $xml = sprintf qq~<?xml version="1.0" encoding="utf-8" ?>
-<otrs_package>
+<otrs_package version="1.0">
     <CVS>\$Id: %s.sopm,v 1.1.1.1 2011/04/15 07:49:58 rb Exp \$</CVS>
     <Name>%s</Name>
     <Version>%s</Version>
@@ -169,6 +202,7 @@ sub execute {
     join( "\n", @xml_parts );
 
     my $fh = IO::File->new( $name . '.sopm', 'w' );
+    $fh->binmode( ':encoding(utf-8)' );
     $fh->print( $xml );
     $fh->close;
 }
@@ -298,7 +332,7 @@ OTRS::OPM::Maker::Command::sopm - Build .sopm file based on metadata
 
 =head1 VERSION
 
-version 0.02
+version 1.06
 
 =head1 CONFIGURATION
 
